#! /usr/bin/perl -w
use DBI qw(:sql_types);
use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser);

sub error {
	  my $msg = shift(@_);
	  print "<h1>ERROR</h1>\n";
	  print "<h2>$msg</h2>\n";
	  print end_html(),"\n";
     exit 1;
} # error

print header;
print start_html(-title=>'CGI Calculate Cost Answer');

# if no form data got sent to this program, we should print
# a message, finish the web page and quit.
if( !param()) {
	print "there is NO stuff in the parameters";
	print end_html(),"\n";
	exit 1;
}

# first we retrieve the value of the 'brand' radio button element.
# All the radio buttons in the group have the same name and the value
# is the contents of the value attribute of the button
# that was selected.
# The $ in front of the variable name means we want to use it as a normal
# variable.
$brand=param("brand");

# now we do pretty much the same thing with the size group of
# radio buttons.
$size=param("size");

# the last thing to get is the value of the howmany text box.
$howmany=param("howmany");

# if we got here, then all the data values are filled in and correct.
# now we will open a database that contains the costs.
# The database has three tables in it.
# The brands table has two fields, the brandid, which is the value used
# in the brand radio button values and the brand which is the
# nicer version of the brand name.
# The sizes table is similar except it contains data on the container sizes.
# The inventory table has 4 attributes.
# It uses the brandid and sizeid as a kind of key.
# For each combination, it also has the cost and an amount.
# The amount is the current inventory of the product.
# We don't use that in this version.

# First we must connect to the database.
# In this example we are using mysql.
# One of the great things about the Perl database interface is
# that changing the database system to Oracle would only require
# us to change the word 'mysql' to 'oracle' in the statement below.
# The rest of the program would work unchanged.
$Dbh = DBI->connect("DBI:mysql:archie-p_teaching","archie-p_nobody",$password) || error("Can't connect to database" . $DBI::errstr);

# now we need to select the data from the inventory table.
# Even though we are not using the amount data, it is simpler to just
# get all of the data from the table.
$s="SELECT * FROM inventory";

# The prepare() function parses the SQL statement and fills in any variables
# we may have put in the statement.
my $sth = $Dbh->prepare($s) || error("prepare failed $DBI::errstr");

# The execute() function hands the SQL statement to the database engine.
# It returns a dataset that has in it all the rows from the database
# that matched the query.
my $rc = $sth->execute || error("Can't execute statement: $DBI::errstr");

# we are going to store the values from the result set as a hash
# A hash is like an array, except that we can use strings as an index
# instead of numbers.
# This is what the param object is. When we give it a string like "brand"
# it gives us the value associated with that string.
# That is why these are also called associative arrays.
my %costs;

# This loop will read the data row by row until the
# end of the data is reached.
# Each of the lines in the loop retrieve the value from the
# current row using the name of the attribute we want.
while ($ref=$sth->fetchrow_hashref) {
      $b=$$ref{brandid};  # get the short version of the brand
      $s=$$ref{sizeid};   # get the short version of the size
      $c=$$ref{cost};     # get the cost 
		$key=$b . "-" . $s; # append into the form 'brand-size'
      $costs{$key} = $c;  # store the cost with the key
} # while

# here is some diagnostic code to print the table
# of brands sizes and costs
#@names= keys %costs;
#foreach $k (@names) {
#	$val=$costs{$k};
#   print "$k --> $val\n";
#}

# The keys in the costs hash are the brand followed by a -
# followed by the size. So we will construct the key from
# the values we got from the form.
# The . (period) operator concatenates strings

$key = $brand . "-" . $size;

$cost = $costs{$key}; # retrieve the cost for our selected choices

# now the total cost
$totalcost = $cost * $howmany;

# to make it prettier, lets figure out nice names for the brands and sizes
# We have information in the database about this as well.
# This query will fetch the one value we want by using the brandid
# in the query so it will search the table to find the row where the
# brandid value matches the brand selected by the user.
$s="SELECT brand FROM brands WHERE brandid = '$brand'";
my $sth = $Dbh->prepare($s) || error("prepare failed $DBI::errstr");
my $rc = $sth->execute || error("Can't execute statement: $DBI::errstr");

#There should be only one but we will loop anyway, just in case
while ($ref=$sth->fetchrow_hashref) {
      $brandout=$$ref{brand};
}

# now the same kind of thing for the size
$s="SELECT size FROM sizes WHERE sizeid = '$size'";
my $sth = $Dbh->prepare($s) || error("prepare failed $DBI::errstr");
my $rc = $sth->execute || error("Can't execute statement: $DBI::errstr");

#There should be only one but we will loop anyway, just in case
while ($ref=$sth->fetchrow_hashref) {
      $sizeout=$$ref{size};
}

#print the charge information
print "<center>\n";
# since '$' is a variable marker, we have to escape it
# using the backslash (\$)
print "$howmany $sizeout containers of $brandout at \$$cost per unit cost \$$totalcost\n";
print "</center>\n";

# all done, print the closing body and html tags
print end_html(),"\n";
exit 1;
